# -*- coding: utf-8 -*-
import os

from django.conf import settings
from django.db import models, transaction


class DeleteFileQuerySet(models.QuerySet):

    @transaction.atomic()
    def delete(self, *args, **kwargs):
        path_list = []
        if settings.THX_CORE_DELETE_FILE_MODEL_DELETE_FILE_ON_DELETE:
            for field in settings.THX_CORE_DELETE_FILE_MODEL_DELETE_FILE_ON_DELETE_FIELDS.get(
                self.model._meta.model_name, []
            ):
                path_queryset = self.values_list(field, flat=True)
                base_path = self.model._meta.get_field(field).storage.base_location
                path_list.extend([os.path.join(base_path, element) for element in path_queryset])

        super().delete(*args, **kwargs)

        if settings.THX_CORE_DELETE_FILE_MODEL_DELETE_FILE_ON_DELETE:
            self.delete_files(path_list)

    @staticmethod
    def delete_files(path_list):
        for path in path_list:
            if os.path.isfile(path):
                os.remove(path)
            folders = path.split(os.sep)[:-1]
            folder_path = os.sep.join(folders)
            if os.path.isdir(folder_path) and not os.listdir(folder_path):
                try:
                    os.rmdir(folder_path)
                except OSError:
                    pass


class DeleteFileManager(models.Manager.from_queryset(DeleteFileQuerySet)):
    pass
